/*
 * $Id: Resolver.java 286 2005-12-03 01:07:16Z davidb $
 *
 * Copyright (c) 2004, 2005 USC Project SRO. All rights reserved.
 * Copyright (c) 2005 VeriSign, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package se.rfc.unbound;

import java.util.Properties;

/**
 * This is the main interface for a "Resolver" module. In the <b>unbound </b>
 * architecture, a resolver module is a component that is responsible for
 * handling the logic of converting a DNS request into a response. Resolver
 * modules form a pipeline or chain of modules that take on this task.
 * 
 * All methods are intended to propagate to the next resolver module in the
 * chain if there is one, unless otherwise stated.
 * 
 * @author davidb
 * @version $Revision: 286 $
 */
public interface Resolver
{

  /**
   * Initialize the resolver module. This is expected to be called once when the
   * module is loaded.
   * 
   * @param config Any configuration information needed or desired to configure
   *          the module. Each module implementation will define its own keys.
   */
  public void init(Properties config) throws Exception;


  /**
   * Prepare the module for unloading. This allows modules to release any
   * resources that they might hold prior to being unloaded.
   */
  public void shutdown();

  
  /**
   * Process a request event.
   * @param event The request event to process.
   */
  public void processRequest(DNSEvent event);
  
  /**
   * Process a response event.
   * @param event The 'response' event to process.
   */
  public void processResponse(DNSEvent event);


  /**
   * @return the next resolver module in the chain, or null if there isn't one.
   */
  public Resolver getNextResolver();

  /**
   * @return the previous resolver module in the chain, or null if there isn't
   *         one.
   */
  public Resolver getPrevResolver();
  
  /**
   * Set the previous resolver module.
   * @param prev The previous module.
   */
  public void setPrevResolver(Resolver prev);
  
  /**
   * Sets the head of the resolution chain. Applications set this so that
   * intermediate modules that generate new, local queries can inject those
   * queries at the head of the resolution pipeline.
   * 
   * @param head The Resolver module that is at the head of the chain.
   */
  public void setResolutionHead(Resolver head);
}